//      attempt at a Trace() implementation macro by [GDS|Entropy]
#include "slopeSnowTex.inc"
#version unofficial megapov 1.21;

global_settings {
 assumed_gamma 1.0
}

camera {
 location <-5, 6,-18>
 up y*image_height right x*image_width
 look_at < 0, 1.5, 0>
 angle 45
} 

light_source {
 0*x
 color rgb <0.2734, 0.5078, 0.7031>  
 area_light
 <100, 0, 0> <0, 0, 100> 
 4, 4                
 adaptive 0          
 jitter              
 translate <0,1000,-2000>   
}

#local RsA = seed(574647);

//      RRand Macro by Chris Huff
#macro RRand(RS, Min, Max) (rand(RS)*(Max-Min) + Min) #end 

//      randVect Macro by [GDS|Entropy]
//------

#macro randVector(Array,ctr,Min,Max,Method)
 #local i=0;
 #while (i<ctr)
  #switch (Method)
   #case (0)    
    #declare Array[i] = <RRand(RsA, Min, Max),RRand(RsA, Min, Max),RRand(RsA, Min, Max)>;
   #break
   #case (1)
    #declare Array[i] = <sin(RRand(RsA, Min, Max)),RRand(RsA, Min, Max),tan(RRand(RsA, Min, Max))>;
   #break    
   #case (2)
    #declare Array[i] = <sin(RRand(RsA, Min, Max)),atanh(RRand(RsA, Min, Max)),tan(RRand(RsA, Min, Max))>;
   #break    
   #case (3)
    #declare Array[i] = <asin(RRand(RsA, Min, Max)),atanh(RRand(RsA, Min, Max)),tan(RRand(RsA, Min, Max))>;
   #break
   #case (4)
    #declare Array[i] = <acosh(RRand(RsA, Min, Max)),asinh(RRand(RsA, Min, Max)),tan(RRand(RsA, Min, Max))>;
   #break
   #case (5)
    #declare Array[i] = <cos(i),i,sin(i)>;
   #break
  #end  
 #set i=i+1;
 #end
#end

//------

//      Textures
#declare IOR = 1.45;
#declare Fade_Distance = 2;
#declare Fade_Power = 3;

#declare InteriorX =
 interior {
  fade_distance Fade_Distance
  fade_power Fade_Power
  ior IOR
  caustics 1.0
 }

#declare FinishX =
 finish {
  diffuse 0.000001
  reflection 0.5
  specular 1
  brilliance 5
  metallic on
  ambient 0.5
  roughness 0.001  
 }

#declare FinishY =
 finish {
  diffuse 0.000001
  reflection 0.5
  specular 1
  brilliance 5
  metallic on  
 }
//------

//      testSphere Macros by [GDS|Entropy]
#macro testSphere0(vect,diameter)
 sphere {
  vect,diameter
  texture {
   pigment {
    color rgb vect
    filter 1
   }
   finish {
    FinishY
   } 
  }
  interior {
   InteriorX
  }     
 }
#end

#macro testSphere1(vect,diameter)
 sphere {
  vect,diameter
  texture {
   pigment {
    color rgb vect
    filter 0
   }
   finish {
    FinishX
   }
  } 
  interior {
   InteriorX
  }     
 }
#end
//------

slopeSnowTex(-1,0.9)

//      snowTest Macro by [GDS|Entropy]
#macro snowTest(ctr,vectorArray,testVector,testObject,radVal,strVal,useInterior)
 #local vectorA = min_extent(testObject);
 #local vectorB = max_extent(testObject);
 blob { 
 #local i=0;
 #while (i<ctr)     
  #if (((vectorArray[i].x <= vectorB.x) & (vectorArray[i].x >= vectorA.x)) & ((vectorArray[i].y <= vectorB.y) & (vectorArray[i].y >= vectorA.y)) & ((vectorArray[i].z <= vectorB.z) & (vectorArray[i].z >= vectorA.z)))
   #if (inside(testObject,vectorArray[i]))
    sphere {
     trace (testObject,vectorArray[i],testVector), radVal, strVal
     //pigment {color rgb vectorArray[i]}
     texture {
      SnowTex0
     }     
    } 
   #end  
  #end                                           
 #set i=i+1;
 #end
 #if (useInterior)
  interior {
   fade_distance 2
   fade_power 3
   ior 1.45
   caustics 1.0 
  } 
 #end 
 }
#end
//------
  //cylinder { -z, +z, RadiusVal, StrengthVal }

//      test objects
#declare objA =
sphere {<0,0,0>,1 scale 2}
/*
torus { 
 1, 0.5 
 translate <0,0,0>
 rotate <90,0,0>
 scale 2
} */

object {
 objA
 //pigment {color rgbf <1,1,1,0.25>}
 texture {
  pigment {
   color rgb <145/255,102/255,32/255>
  }
 }
 finish { 
  ambient 0 
  diffuse 1 
  specular 0.5 
  roughness 0.01 
  reflection 0.03 
 }
 normal { 
  wrinkles 0.2 
  scale 0.5 
 } 
}

#declare objB =
cone { 
 <0, 1, 0>, 0.0, <0, -1, 0>, 1.0 
 scale 2
}

#declare objC = 
quartic {
 < 0.0,  0.0,  0.0,  0.0,  1.0,  0.0,  0.0,  0.0,  0.0,  0.01,
   0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,
   0.0,  0.0,  0.0,  1.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,
   0.0,  0.0,  0.01, 0.0, -0.01 >
  bounded_by { sphere { <0, 0, 0>, 2 } }
  scale 2
}
//------

#declare Sp = 5000;
#declare randVectArray = array [Sp];
randVector(randVectArray,Sp,-4,4,0)

#declare test = union{snowTest(Sp,randVectArray,<0,12,0>,objA,1,1,yes)};

object {
 test
}
